package sync4j.zimbra.admin;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import sync4j.framework.engine.source.ContentType;
import sync4j.framework.engine.source.SyncSourceInfo;

import sync4j.zimbra.engine.source.ZimbraCalendarSyncSource;
import sync4j.zimbra.engine.source.ZimbraContactSyncSource;
import sync4j.zimbra.engine.source.ZimbraSyncSource;

import sync4j.syncadmin.AdminException;
import sync4j.syncadmin.ui.SourceManagementPanel;


import java.net.URL;


/**
 * This class implements the configuration panel for a ZimbraSyncSource 
 */
public class ZimbraSyncSourceConfigPanel extends SourceManagementPanel implements
        Serializable {

    // ----------------------------------------------------------------- Constants

    public static final String NAME_ALLOWED_CHARS =
            "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_./";
    public static final String PORT_ALLOWED_CHARS =
            "0123456789";
                                       
    public static final String VCARD_TYPE = "text/x-vcard";
    public static final String VCARD_TYPES = "text/x-vcard,text/vcard";
    public static final String VCARD_VERSIONS = "2.1, 3.0";
    public static final String ICAL_TYPE = "text/x-vcalendar";
    public static final String ICAL_TYPES = "text/x-vcalendar";
    public static final String ICAL_VERSIONS = "1.0";
    public static final String SIFC_TYPE = "text/x-s4j-sifc";
    public static final String SIFC_TYPES = "text/x-s4j-sifc";
    public static final String SIFC_VERSIONS = "1.0";
    public static final String SIFE_TYPE = "text/x-s4j-sife";
    public static final String SIFE_TYPES = "text/x-s4j-sife";
    public static final String SIFE_VERSIONS = "1.0";
    public static final String SIFN_TYPE = "text/x-s4j-sifn";
    public static final String SIFN_TYPES = "text/x-s4j-sifn";
    public static final String SIFN_VERSIONS = "1.0";
    public static final String SIFT_TYPE = "text/x-s4j-sift";
    public static final String SIFT_TYPES = "text/x-s4j-sift";
    public static final String SIFT_VERSIONS = "1.0";

    // -------------------------------------------------------------- Private data
    /**
     * label for the panel's name
     */
    private JLabel panelName = new JLabel();

    private JLabel nameLabel = new JLabel();
    private JTextField nameValue = new JTextField();

    private JLabel sourceUriLabel = new JLabel();
    private JTextField sourceUriValue = new JTextField();

    private JLabel zimbraUrlLabel = new JLabel();
    private JTextField zimbraUrlValue = new JTextField();

    private JLabel conversionTypeLabel = new JLabel();
    private JComboBox conversion = new JComboBox();

    private JButton confirmButton = new JButton();

    // ----------------------------------------------------------- Constructors

    public ZimbraSyncSourceConfigPanel() {
        init();
    }

    // ----------------------------------------------------------- Private methods
    /**
     * Init the panel
     */

    private void init() {

        int h = 20;

        int y = 5;
        int dy = 30;
        int x1 = 14;
        int x2 = 170;
        int w1 = 150;
        int w2 = 350;

        // set layout
        this.setLayout(null);

        // set properties of label, position and border
        //  referred to the title of the panel
        TitledBorder titledBorder1;
        titledBorder1 = new TitledBorder("");

        panelName.setFont(titlePanelFont);
        panelName.setText("Edit Zimbra SyncSource");
        panelName.setBounds(new Rectangle(x1, y, 316, 28));
        panelName.setAlignmentX(SwingConstants.CENTER);
        panelName.setBorder(titledBorder1);

        y += 55;

        sourceUriLabel.setText("Source URI: ");
        sourceUriLabel.setFont(defaultFont);
        sourceUriLabel.setBounds(new Rectangle(x1, y, w1, h));
        sourceUriValue.setFont(defaultFont);
        sourceUriValue.setBounds(new Rectangle(x2, y, w2, h));
        sourceUriValue.setText("contacts");

        y += dy;

        nameLabel.setText("Name: ");
        nameLabel.setFont(defaultFont);
        nameLabel.setBounds(new Rectangle(x1, y, w1, h));
        nameValue.setFont(defaultFont);
        nameValue.setBounds(new Rectangle(x2, y, w2, h));
        nameValue.setText("contacts");

        y += dy;

        zimbraUrlLabel.setText("Zimbra Server Url: ");
        zimbraUrlLabel.setFont(defaultFont);
        zimbraUrlLabel.setBounds(new Rectangle(x1, y, w1, h));
        zimbraUrlValue.setFont(defaultFont);
        zimbraUrlValue.setBounds(new Rectangle(x2, y, w2, h));
        zimbraUrlValue.setText("[i.e http://localhost/zimbra]");

        y += dy;

        conversionTypeLabel.setText("Type: ");
        conversionTypeLabel.setFont(defaultFont);
        conversionTypeLabel.setBounds(new Rectangle(x1, y, w1, h));
        conversion.setFont(defaultFont);
        conversion.setBounds(new Rectangle(x2, y, 80, h));

        y += 60;

        confirmButton.setFont(defaultFont);
        confirmButton.setText("Add");
        confirmButton.setBounds(170, y, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                try {
                    validateValues();
                    getValues();

                    if (getState() == STATE_INSERT) {
                        ZimbraSyncSourceConfigPanel.this.actionPerformed(new
                                ActionEvent(ZimbraSyncSourceConfigPanel.this,
                                        ACTION_EVENT_INSERT,
                                        event.getActionCommand()));
                    } else {
                        ZimbraSyncSourceConfigPanel.this.actionPerformed(new
                                ActionEvent(ZimbraSyncSourceConfigPanel.this,
                                        ACTION_EVENT_UPDATE,
                                        event.getActionCommand()));
                    }
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });

        // add all components to the panel
        this.add(panelName, null);
        this.add(sourceUriLabel, null);
        this.add(sourceUriValue, null);
        this.add(nameLabel, null);
        this.add(nameValue, null);
        this.add(zimbraUrlLabel, null);
        this.add(zimbraUrlValue, null);
        this.add(conversionTypeLabel, null);
        this.add(conversion, null);

        this.add(confirmButton, null);
    }

    public void updateForm() {

        if (!(getSyncSource() instanceof ZimbraSyncSource)) {
            error("This is not a ZimbraSyncSource!" +
                    "Unable to process SyncSource values.");
            return;
        }

        if (getState() == STATE_INSERT) {
            confirmButton.setText("Add");
        } else if (getState() == STATE_UPDATE) {
            confirmButton.setText("Save");
        }

        ZimbraSyncSource syncSource = (ZimbraSyncSource) getSyncSource();

        sourceUriValue.setText(syncSource.getSourceURI());
        nameValue.setText(syncSource.getName());

        String zimbraUrl = syncSource.getZimbraUrl();
        if (zimbraUrl == null || zimbraUrl.equals("")) {
            zimbraUrl = "[i.e http://localhost/zimbra]";
        }
        zimbraUrlValue.setText(zimbraUrl);

        conversion.removeAllItems();

        if (syncSource instanceof ZimbraCalendarSyncSource) {
            conversion.addItem("SIF-E");
            conversion.addItem("iCal");
        } else if (syncSource instanceof ZimbraContactSyncSource) {
            conversion.addItem("SIF-C");
            conversion.addItem("vCard");
        }

        if ((syncSource instanceof ZimbraCalendarSyncSource) ||
                (syncSource instanceof ZimbraContactSyncSource)) {

            String cType = (syncSource).getType();

            if (VCARD_TYPE.equals(cType) || ICAL_TYPE.equals(cType)) {
                conversion.setSelectedIndex(1);
            } else {
                conversion.setSelectedIndex(0);
            }
        }

        if (syncSource.getSourceURI() != null) {
            sourceUriValue.setEditable(false);
        }
    }


    // ----------------------------------------------------------- Private methods

    /**
     * Checks if the values provided by the user are all valid. In caso of errors,
     * a IllegalArgumentException is thrown.
     *
     * @throws IllegalArgumentException if:
     *                                  <ul>
     *                                  <li>name, uri, type or directory are empty (null or zero-length)
     *                                  <li>the types list length does not match the versions list length
     *                                  </ul>
     */
    private void validateValues() throws IllegalArgumentException {
        String value;

        value = nameValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Name' cannot be empty. " +
                    "Please provide a SyncSource name.");
        }

        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new IllegalArgumentException("Only the following characters are " +
                    "allowed for field 'Name': \n" +
                    NAME_ALLOWED_CHARS);
        }

        value = zimbraUrlValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Zimbra Server Url' cannot be empty. " +
                    "Please provide a valid url.");
        }

        try {
            new URL(value);
        } catch (Exception ex) {
            throw new IllegalArgumentException("Zimbra Server Url isn't a valid url. " +
                    "Please provide a valid url.");
        }


        value = sourceUriValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Source URI' cannot be empty. " +
                    "Please provide a SyncSource URI.");
        }

    }

    /**
     * Set syncSource properties with the values provided by the user.
     */
    private void getValues() {

        ZimbraSyncSource syncSource = (ZimbraSyncSource) getSyncSource();

        syncSource.setSourceURI(sourceUriValue.getText().trim());
        syncSource.setName(nameValue.getText().trim());

        syncSource.setZimbraUrl(zimbraUrlValue.getText().trim());

        StringTokenizer types = null;
        StringTokenizer versions = null;

        if (conversion.getSelectedIndex() == 0) {
            syncSource.setEncode(true);

            if (syncSource instanceof ZimbraCalendarSyncSource) {
                syncSource.setType(SIFE_TYPE);
                types = new StringTokenizer(SIFE_TYPES, ",");
                versions = new StringTokenizer(SIFE_VERSIONS, ",");
            } else if (syncSource instanceof ZimbraContactSyncSource) {
                syncSource.setType(SIFC_TYPE);
                types = new StringTokenizer(SIFC_TYPES, ",");
                versions = new StringTokenizer(SIFC_VERSIONS, ",");
            }

        } else if (syncSource instanceof ZimbraCalendarSyncSource) {
            syncSource.setEncode(false);
            syncSource.setType(ICAL_TYPE);
            types = new StringTokenizer(ICAL_TYPES, ",");
            versions = new StringTokenizer(ICAL_VERSIONS, ",");
        } else if (syncSource instanceof ZimbraContactSyncSource) {
            syncSource.setEncode(false);
            syncSource.setType(VCARD_TYPE);
            types = new StringTokenizer(VCARD_TYPES, ",");
            versions = new StringTokenizer(VCARD_VERSIONS, ",");
        }

        ContentType[] contentTypes = new ContentType[types.countTokens()];

        for (int i = 0, l = contentTypes.length; i < l; ++i) {
            try {
                contentTypes[i] = new ContentType(types.nextToken().trim(),
                        versions.nextToken().trim());
            } catch (NullPointerException e) {
                return;
            }
        }

        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));

    }


    /**
     * @param msg String
     */
    private void error(String msg) {
        JOptionPane.showMessageDialog(null, msg, "Error",
                JOptionPane.ERROR_MESSAGE);
    }

}
