package sync4j.zimbra.items.manager;

import java.util.logging.Logger;


import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.exception.ZimbraAccessException;
import sync4j.zimbra.util.Def;
import sync4j.zimbra.util.ZimbraWrapper;

import java.util.HashMap;

/**
 * This is a base class for all managers.
 */
public class EntityManager {

    public static final String PROPERTY_NAME_SERVER_URL = "zimbraUrl";
    public static final String PROPERTY_NAME_USER_NAME = "username";
    public static final String PROPERTY_NAME_PASSWORD = "password";
    public static final String PHONE = "phone";

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    /**
     * Wrapper to access to Zimbra Soap services
     */
    public ZimbraWrapper zimbraWrapper = null;

    // ------------------------------------------------------------- Costructors

    /**
     * Creates a new EntityManager and open a session on Zimbra Soap services.
     *
     * @param properties
     * @throws EntityException
     */
    public EntityManager(HashMap properties) throws EntityException {

        String serverUrl = (String) properties.get(PROPERTY_NAME_SERVER_URL);
        String username = (String) properties.get(PROPERTY_NAME_USER_NAME);
        String password = (String) properties.get(PROPERTY_NAME_PASSWORD);
        String phone = (String) properties.get(PHONE);

        // open Zimbra connection
        zimbraWrapper = new ZimbraWrapper();
        zimbraWrapper.setPhone(phone);

        try {
            zimbraWrapper.createSession(serverUrl,
                    username,
                    password);
        } catch (ZimbraAccessException e) {
            throw new EntityException("Error connecting to Zimbra [" +
                    e.getMessage() + "]", e);
        }

    }

    // ---------------------------------------------------------- Public Methods

    /**
     * Releases theZimbra connection.
     *
     * @throws EntityException
     */
    public void releaseSession() throws EntityException {

        // close Zimbra connection
        try {
            zimbraWrapper.releaseSession();
        } catch (ZimbraAccessException e) {
            throw new EntityException("Error closing Zimbra connection [" +
                    e.getMessage() + "]", e);
        }
    }


    // --------------------------------------------------------- Private Methods
}

