package sync4j.zimbra.util;

import java.text.ParseException;

public class ICalParser
{


    public static final String DATETIME_START = "DTSTART";
    public static final String DATETIME_END   = "DTEND";


    private ICalParser()
    {

    }

    public static String remove13(String iCal)
    {
        byte[] iCalBytes = iCal.getBytes();
        byte[] iCalBytes13Free = new byte[iCalBytes.length];
        int length=0;
        for(int i=0; i<iCalBytes.length; i++)
            if(iCalBytes[i]!=13) iCalBytes13Free[length++]=iCalBytes[i];
        return new String(iCalBytes13Free, 0, length);
    }

    public static String getEndDate(String iCal)
    throws ParseException
    {
        String endDate;
        int index = iCal.indexOf(DATETIME_END);
        if(index<0) return null;
        index += DATETIME_END.length();
        if(iCal.charAt(index)!=':') return null;
        index++;
        endDate = iCal.substring(index, index+iCal.substring(index).indexOf("\n"));
        if(endDate.length()<15) return endDate;
        endDate = adjustEndDate(endDate);
        return endDate;
    }

    public static String adjustDates(String iCal)
    throws ParseException
    {
        int index = iCal.indexOf(DATETIME_END);
        if(index<0) return null;
        index += DATETIME_END.length();
        if(iCal.charAt(index)!=':')
            throw new ParseException("Unparsable iCal!", index+1);
        index++;
        String endDate = getEndDate(iCal);
        return iCal.substring(0, index)+endDate+iCal.substring(index+endDate.length());

    }

    private static String adjustEndDate(String sDate)
    throws ParseException
    {
        if(sDate.length()<9) return sDate;
        try
        {
            int hour = Integer.parseInt(sDate.substring(9, 11));
            if(hour>=24) return sDate.substring(0,9) + "235959Z";
            return sDate;
        }
        catch(Exception e)
        {
            throw new ParseException("Invalid date description: " + sDate, 10);
        }
    }


}
