package sync4j.zimbra.util;

import sync4j.foundation.pdi.event.RecurrencePattern;
import sync4j.zimbra.exception.EntityException;
import zimbra.soap.util.RecurrenceDetails;

import java.util.HashMap;


public class RecurrencePatternExt extends RecurrencePattern
{

    public RecurrencePatternExt(short reccurrenceType,
                                int interval,
                                short monthOfYear,
                                short dayOfMonth,
                                short dayOfweekMask,
                                short instance,
                                String startDatePattern,
                                String endDatePattern,
                                boolean noEndDate)
    {
        super(reccurrenceType,
              interval,
              monthOfYear,
              dayOfMonth,
              dayOfweekMask,
              instance,
              startDatePattern,
              endDatePattern,
              noEndDate);
    }

    public static RecurrencePattern getRecurrencePattern(RecurrenceDetails recurrenceDetails)
    throws EntityException
    {
        if(recurrenceDetails ==null) return null;

        String patternStartDate = getPatternStartDate(recurrenceDetails);
        String patternEndDate   = getPatternEndDate(recurrenceDetails);
        String timeZone         = null;
        //int occurrences         = 0;
        boolean noEndDate       = getNoEndDate(recurrenceDetails);
        short recurrenceType    = 0;
        int interval            = 1;
        short daysOfWeekMask    = 0;
        short monthOfYear       = 0;
        short dayOfMonth        = 0;
        short instance          = 0;

        String temp;
        Object[] objs;

        try
        {
            /*temp = (String)recurrenceDetails.get(RecurrenceDetails.K_RULE_COUNT);
            if(temp != null)
            {
                try{occurrences = Integer.parseInt(temp);}
                catch(Exception e){}
                noEndDate  =  false;
            }*/

            objs = (Object[])recurrenceDetails.get(RecurrenceDetails.K_RULE_BYMONTHDAY);
            if(objs != null && objs.length>0)
            {
                try{dayOfMonth = Short.parseShort((String)objs[0]);}
                catch(Exception e) {dayOfMonth = 0;}
            }

            objs = (Object[])recurrenceDetails.get(RecurrenceDetails.K_RULE_BYDAY);
            if(objs != null)
            {
                HashMap hm;
                for(int i=0; i<objs.length; i++)
                {
                    hm = (HashMap) objs[i];
                    String day = (String) hm.get(RecurrenceDetails.K_RULE_BYDAY_DAY);
                    String ordwk= (String) hm.get(RecurrenceDetails.K_RULE_BYDAY_ORDWK);
                    if(ordwk!=null)
                        try{instance = Short.parseShort(ordwk);}
                        catch(Exception e){instance = 0;}
                    if(day == null) continue;
                    if("SU".equals(day)) daysOfWeekMask+=1;
                    if("MO".equals(day)) daysOfWeekMask+=2;
                    if("TU".equals(day)) daysOfWeekMask+=4;
                    if("WE".equals(day)) daysOfWeekMask+=8;
                    if("TH".equals(day)) daysOfWeekMask+=16;
                    if("FR".equals(day)) daysOfWeekMask+=32;
                    if("SA".equals(day)) daysOfWeekMask+=64;

                }
            }

            temp = (String)recurrenceDetails.get(RecurrenceDetails.K_RULE_FREQUENCY);
            if(temp!=null)
            {
                if("DAI".equals(temp)) recurrenceType = (short)0;
                if("WEE".equals(temp))
                {
                    recurrenceType = (short)1;
                    if(daysOfWeekMask == 0)
                        try
                        {
                            daysOfWeekMask =
                            UtilityCalendar.getDayOfWeek(patternStartDate,
                                                         timeZone);
                        }
                        catch(Exception e)
                        {
                            throw new EntityException("Error getting day of the week: " +
                                                      e.getMessage(), e);
                        }
                }
                if("MON".equals(temp))
                {
                    if(daysOfWeekMask == 0)
                        try
                        {
                            if ( patternStartDate != null )
                            {
                            dayOfMonth =
                                UtilityCalendar.getDayOfMonth(patternStartDate,
                                                              timeZone);
                            }
                         }
                         catch (Exception e)
                         {
                            throw new EntityException("Error getting day of the month: "+
                                                      e.getMessage(), e);
                         }
                    if(dayOfMonth > 0)
                        recurrenceType = (short)2;
                    else recurrenceType = (short)3;
                }
                if("YEA".equals(temp))
                {
                    if(daysOfWeekMask == 0)
                        try
                        {
                            if ( patternStartDate != null )
                            {
                            dayOfMonth =
                                UtilityCalendar.getDayOfMonth(patternStartDate,
                                                              timeZone);
                            }
                         }
                         catch (Exception e)
                         {
                            throw new EntityException("Error getting day of the month!", e);
                         }
                    if(dayOfMonth > 0)
                        recurrenceType = (short)5;
                    else recurrenceType = (short)6;

                }

            }
            temp = (String)recurrenceDetails.get(RecurrenceDetails.K_RULE_INTERVAL);
            if(temp != null) interval = Integer.parseInt(temp);

            objs = (Object[])recurrenceDetails.get(RecurrenceDetails.K_RULE_BYMONTH);
            if(objs != null && objs.length>0)
            {
                try{monthOfYear = Short.parseShort((String)objs[0]);}
                catch(Exception e){monthOfYear = 0;}
            }

            return new RecurrencePatternExt(
                    recurrenceType,
                    interval,
                    monthOfYear,
                    dayOfMonth,
                    daysOfWeekMask,
                    instance,
                    patternStartDate,
                    patternEndDate,
                    noEndDate);
        }
        catch(Exception e)
        {
            throw new EntityException(
            "Conversion from RecurreceDetails to RecurrencePattern failed: "
            + e, e);
        }
    }

    private static boolean getNoEndDate(RecurrenceDetails recurrenceDetails)
    {
        return
        recurrenceDetails.get(RecurrenceDetails.K_RULE_DATETIME_END) == null &&
        recurrenceDetails.get(RecurrenceDetails.K_RULE_COUNT) == null;
    }

    private static String getPatternStartDate(RecurrenceDetails recurrenceDetails)
    throws EntityException
    {
        String patternStartDate;
        String timeZone = null;
        Object obj;

        obj = recurrenceDetails.get(RecurrenceDetails.K_DATETIME_START);
        if(obj == null) return null;
        patternStartDate = obj.toString();
        obj = recurrenceDetails.get(RecurrenceDetails.K_TIMEZONE);
        if(obj != null) timeZone = obj.toString();
        try
        {
            if(patternStartDate.length()<0) patternStartDate +="T000000";
            patternStartDate =
            UtilityCalendar.covertLocalDateToGMT(patternStartDate,
                                                 timeZone);
        }
        catch(Exception e)
        {
            throw new EntityException(e);
        }
        return patternStartDate;
    }

    private static String getPatternEndDate(RecurrenceDetails recurrenceDetails)
    throws EntityException
    {
        String patternEndDate;
        String timeZone = null;
        Object obj;

        obj = recurrenceDetails.get(RecurrenceDetails.K_RULE_DATETIME_END);
        if(obj == null) return null;
        patternEndDate = obj.toString();
        obj = recurrenceDetails.get(RecurrenceDetails.K_TIMEZONE);
        if(obj != null) timeZone = obj.toString();
        try
        {
            if(patternEndDate.length()<0) patternEndDate +="T235959";
            patternEndDate =
            UtilityCalendar.covertLocalDateToGMT(patternEndDate,
                                                 timeZone);
        }
        catch(Exception e)
        {
            throw new EntityException(e);
        }
        return patternEndDate;
    }

}
