package sync4j.zimbra.util;

import java.util.*;
import java.util.logging.Logger;

import sync4j.foundation.pdi.contact.*;


public class UtilityContact {


    /**
     * Returns the business phone from
     * foundation contact object
     * <ul>
     * <li>1 BusinessTelephoneNumber</li>
     * <li>2 Business2TelephoneNumber</li>
     * <li>3 BusinessFaxNumber</li>
     * </ul>
     *
     * @param contact Contact
     * @param type    String
     * @return String
     */
    public static String[] getBusinessPhonesFromContact(Contact contact,
                                                        String type) {
        String phone;
        Phone businessPhone;
        List businessPhones;
        List businessPhonesOfTypeList;

        Logger log = Logger.getLogger(Def.LOGGER_NAME);

        if (!Def.BUSINESS_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.BUSINESS_2_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.BUSINESS_FAX_NUM.equalsIgnoreCase(type))
            type = null;

        if (contact.getBusinessDetail() == null) return new String[0];
        businessPhones = contact.getBusinessDetail().getPhones();
        if (businessPhones == null || businessPhones.size() <= 0) return new String[0];
        businessPhonesOfTypeList = new LinkedList();
        for (int i = 0; i < businessPhones.size(); i++) {
            try {
                businessPhone = (Phone) businessPhones.get(i);
            } catch (Exception e) {
                continue;
            }
            if (type == null || (businessPhone.getPhoneType() != null &&
                    businessPhone.getPhoneType().equalsIgnoreCase(type))) {
                phone = (String) businessPhone.getPropertyValue();
                if (phone != null && !"".equals(phone) && !" ".equals(phone))
                    businessPhonesOfTypeList.add(phone);
            }
        }
        return (String[]) businessPhonesOfTypeList.toArray(new String[0]);
    }


    /**
     * returns the personal phone from
     * foundation contact object
     * <ul>
     * <li>1 OtherTelephoneNumber</li>
     * <li>2 MobileTelephoneNumber</li>
     * <li>3 HomeTelephoneNumber</li>
     * <li>4 Home2TelephoneNumber</li>
     * <li>5 HomeFaxNumber</li>
     * </ul>
     *
     * @param contact Contact
     * @param type    String
     * @return String
     */
    public static String[] getPersonalPhonesFromContact(Contact contact,
                                                        String type) {

        String phone;
        Phone personalPhone;
        List personalPhones;
        List personalPhonesOfTypeList;

        if (!Def.PERSONAL_OTHER_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.PERSONAL_HOME_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.PERSONAL_HOME_2_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.PERSONAL_MOBILE_PHONE_NUM.equalsIgnoreCase(type) &&
                !Def.PERSONAL_HOME_FAX_NUMBER.equalsIgnoreCase(type) &&
                !Def.PERSONAL_OTHER3_FAX_NUMBER.equalsIgnoreCase(type) &&
                !Def.PERSONAL_OTHER2_FAX_NUMBER.equalsIgnoreCase(type) &&
                !Def.PERSONAL_OTHER_FAX_NUMBER.equalsIgnoreCase(type))
            type = null;

        Logger log = Logger.getLogger(Def.LOGGER_NAME);

        if (contact.getPersonalDetail() == null) return new String[0];
        personalPhones = contact.getPersonalDetail().getPhones();
        if (personalPhones == null || personalPhones.size() <= 0) return new String[0];
        personalPhonesOfTypeList = new LinkedList();
        for (int i = 0; i < personalPhones.size(); i++) {
            try {
                personalPhone = (Phone) personalPhones.get(i);
            } catch (Exception e) {
                continue;
            }
            if (type == null || personalPhone.getPhoneType().equalsIgnoreCase(type)) {
                phone = (String) personalPhone.getPropertyValue();
                if (phone != null && !"".equals(phone) && !" ".equals(phone))
                    personalPhonesOfTypeList.add(phone);
            }
        }

        return (String[]) personalPhonesOfTypeList.toArray(new String[0]);
    }

    public static String getPersonalPhoneFromContact(Contact contact, String type, int index) {
        try {
            return getPersonalPhonesFromContact(contact, type)[index];
        } catch (Exception e) {
            return null;
        }

    }


    /**
     * returns the emails of a type from
     * foundation contact object
     * <ul>
     * <li>1 Email1Address - Home</li>
     * <li>2 Email2Address - Other</li>
     * <li>3 Email3Address - Work</li>
     * </ul>
     *
     * @param contact Contact
     * @param type    String
     * @return String
     */
    public static String[] getEmailsFromContact(Contact contact,
                                                String type) {

        String strEmail;
        List emails;
        List emailsOfType;
        Email email;

        if (Def.PERSONAL_EMAIL.equalsIgnoreCase(type) ||
                Def.PERSONAL_EMAIL_1_ADDRESS.equalsIgnoreCase(type) ||
                Def.PERSONAL_EMAIL_2_ADDRESS.equalsIgnoreCase(type))
            emails = contact.getPersonalDetail().getEmails();
        else if (Def.BUSINESS_EMAIL_3_ADDRESS.equalsIgnoreCase(type))
            emails = contact.getBusinessDetail().getEmails();
        else
            return new String[0];
        if (emails == null || emails.size() <= 0) return new String[0];
        emailsOfType = new LinkedList();

        for (int i = 0; i < emails.size(); i++) {
            try {
                email = (Email) emails.get(i);
            } catch (Exception e) {
                continue;
            }
            if (Def.PERSONAL_EMAIL.equalsIgnoreCase(type) ||
                    email.getEmailType().equalsIgnoreCase(type)) {
                strEmail = (String) email.getPropertyValue();
                if (strEmail != null && !"".equals(strEmail) && !" ".equals(strEmail))
                    emailsOfType.add(strEmail);
            }
        }
        return (String[]) emailsOfType.toArray(new String[0]);
    }


    /**
     * @param contact Contact
     * @return String
     */
    public static String getNote1FromFoundation(sync4j.foundation.pdi.contact.Contact contact) {

        String sNote = null;

        List notes = contact.getNotes();

        if (notes != null && notes.size() > 0) {
            sNote = (String) ((Note) notes.get(0)).getPropertyValue();
        }

        return sNote;
    }


    /**
     * returns the title from
     * foundation contact object
     * <ul>
     * <li>1 JobTitle</li>
     * </ul>
     *
     * @param contact Contact
     * @param type_e  int
     * @return String
     */
    public static String getBusinessTitleFromFoundation(sync4j.foundation.pdi.contact.Contact contact,
                                                        int type_e) {

        String title = null;
        Title businessTitle;

        String type = Def.BUSINESS_TITLE;
        if (type_e == 1) {
            type = Def.BUSINESS_TITLE;
        }

        BusinessDetail businessDetail = contact.getBusinessDetail();

        List titles = businessDetail.getTitles();
        int size = 0;
        if (titles != null) {
            size = titles.size();
        }

        for (int i = 0; i < size; i++) {
            try {
                businessTitle = (Title) titles.get(i);
            } catch (NullPointerException e) {
                return null;
            }
            if (businessTitle.getTitleType().equalsIgnoreCase(type)) {
                title = (String) businessTitle.getPropertyValue();
                break;
            }
        }

        return title;
    }

    /**
     * set the title in the foundation contact
     * object.
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void setBusinessTitleInFoundation(Contact contact,
                                                    Object newValue) {
        Title businessTitle = new Title();
        String strBusinessTitle;
        if (newValue != null)
            strBusinessTitle = newValue.toString();
        else
            strBusinessTitle = "";
        businessTitle.setTitleType(Def.BUSINESS_TITLE);
        businessTitle.setPropertyValue(strBusinessTitle);
        contact.getBusinessDetail().addTitle(businessTitle);
    }

    /**
     * set the business email in the foundation
     * contact object.
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void addEmailInContact(Contact contact,
                                         String type,
                                         Object newValue) {

        if (newValue == null) return;
        List emailsList;
        String strEmail = newValue.toString();
        Email email = new Email();
        email.setPropertyValue(strEmail);
        if (!Def.PERSONAL_EMAIL_1_ADDRESS.equals(type) &&
                !Def.PERSONAL_EMAIL_2_ADDRESS.equals(type) &&
                !Def.BUSINESS_EMAIL_3_ADDRESS.equals(type))
            type = Def.PERSONAL_EMAIL_1_ADDRESS;
        email.setEmailType(type);
        if (Def.BUSINESS_EMAIL_3_ADDRESS.equals(type))
            emailsList = contact.getBusinessDetail().getEmails();
        else
            emailsList = contact.getPersonalDetail().getEmails();
        if (emailsList == null) emailsList = new LinkedList();

        emailsList.add(email);

        if (Def.BUSINESS_EMAIL_3_ADDRESS.equals(type))
            contact.getBusinessDetail().setEmails(emailsList);
        else
            contact.getPersonalDetail().setEmails(emailsList);
    }

    /**
     * set the business email in the foundation
     * contact object.
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void setNote1InFoundation(Contact contact,
                                            Object newValue) {
        String sNote = "";
        Note note = new Note();
        if (newValue != null) sNote = newValue.toString();
        note.setPropertyValue(sNote);
        note.setNoteType(Def.NOTE_TYPE);
        contact.addNote(note);
    }

    public static void setNameInContact(Contact contact,
                                        Object firstName,
                                        Object lastName,
                                        Object middleName) {
        Name name = contact.getName();
        String strFirstName, strLastName;
        strFirstName = strLastName = "";
        if (firstName != null) strFirstName = firstName.toString();
        if (lastName != null) strLastName = lastName.toString();
        // this property set the <fileAs> tag
        name.getDisplayName().setPropertyValue(strLastName + "," +
                strFirstName);
        name.getFirstName().setPropertyValue(firstName);
        name.getMiddleName().setPropertyValue(lastName);
        name.getLastName().setPropertyValue(middleName);

    }

    /**
     * sets the business phone in the
     * foundation contact object
     * <p/>
     * <ul>
     * <li>1 BusinessTelephoneNumber</li>
     * <li>2 Business2TelephoneNumber</li>
     * <li>3 BusinessFaxNumber</li>
     * </ul>
     *
     * @param contact  Contact
     * @param newValue String
     * @param type_e   int
     */
    public static void setBusinessPhoneInFoundation(sync4j.foundation.pdi.contact.Contact contact,
                                                    String newValue,
                                                    int type_e) {

        String type = Def.BUSINESS_PHONE_NUM;
        if (type_e == 1) {
            type = Def.BUSINESS_PHONE_NUM;
        } else if (type_e == 2) {
            type = Def.BUSINESS_2_PHONE_NUM;
        } else if (type_e == 3) {
            type = Def.BUSINESS_FAX_NUM;
        }

        Phone businessPhone = new Phone();

        businessPhone.setPhoneType(type);
        businessPhone.setPropertyValue(newValue);

        contact.getBusinessDetail().addPhone(businessPhone);
    }

    /**
     * stes the web page in the
     * foundation contact object
     *
     * @param contact  Contact
     * @param newValue String
     */
    public static void addWebPageInContact(Contact contact,
                                           String type,
                                           Object newValue) {
        if (newValue == null) return;
        List webPagesList;
        String strWebPage = newValue.toString();
        WebPage webPage = new WebPage();
        webPage.setPropertyValue(strWebPage);
        if (!Def.BUSINESS_WEBPAGE.equals(type) &&
                !Def.PERSONAL_WEBPAGE.equals(type) &&
                !Def.OTHER_WEBPAGE.equals(type))
            type = Def.PERSONAL_WEBPAGE;
        webPage.setWebPageType(type);
        if (Def.BUSINESS_WEBPAGE.equals(type))
            webPagesList = contact.getBusinessDetail().getWebPages();
        else
            webPagesList = contact.getPersonalDetail().getWebPages();
        if (webPagesList == null) webPagesList = new LinkedList();

        webPagesList.add(webPage);
        if (Def.BUSINESS_WEBPAGE.equals(type))
            contact.getBusinessDetail().setWebPages(webPagesList);
        else
            contact.getPersonalDetail().setWebPages(webPagesList);
    }

    /**
     * returns the web page from
     * foundation contact object
     *
     * @param contact Contact
     * @return String
     */
    public static String[] getWebPagesOfTypeFromContact(Contact contact,
                                                        String type) {

        List webPagesOfTypeList = new LinkedList();
        List webPagesList = null;
        if (Def.BUSINESS_WEBPAGE.equals(type))
            webPagesList = contact.getBusinessDetail().getWebPages();
        else if (Def.PERSONAL_WEBPAGE.equals(type) ||
                Def.OTHER_WEBPAGE.equals(type))
            webPagesList = contact.getPersonalDetail().getWebPages();
        if (webPagesList == null || webPagesList.size() == 0) return new String[0];

        int size = webPagesList.size();

        WebPage webPage;
        for (int i = 0; i < size; i++) {
            try {
                webPage = (WebPage) webPagesList.get(i);
            } catch (Exception e) {
                continue;
            }
            if (webPage.getWebPageType().equalsIgnoreCase(type))
                webPagesOfTypeList.add(webPage.getPropertyValueAsString());
        }
        return (String[]) webPagesOfTypeList.toArray(new String[0]);
    }

    /**
     * stes the personal phone in the
     * foundation contact object
     * <p/>
     * <ul>
     * <li>1 OtherTelephoneNumber</li>
     * <li>2 MobileTelephoneNumber</li>
     * <li>3 HomeTelephoneNumber</li>
     * <li>4 Home2TelephoneNumber</li>
     * <li>5 HomeFaxNumber</li>
     * </ul>
     *
     * @param contact  Contact
     * @param newPhone String
     * @param type     String
     */
    public static void addPhoneInContact(Contact contact,
                                         String type,
                                         Object newPhone) {

        List phonesList;
        int bigType;

        if (newPhone == null) return;

        if (Def.PERSONAL_HOME_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.PERSONAL_HOME_2_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.PERSONAL_HOME_FAX_NUMBER.equalsIgnoreCase(type) ||
                Def.PERSONAL_MOBILE_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.PERSONAL_OTHER_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.PERSONAL_OTHER_FAX_NUMBER.equalsIgnoreCase(type)) {
            if (contact.getPersonalDetail() == null) return;
            phonesList = contact.getPersonalDetail().getPhones();
            bigType = 1;
        } else if (Def.BUSINESS_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.BUSINESS_2_PHONE_NUM.equalsIgnoreCase(type) ||
                Def.BUSINESS_FAX_NUM.equalsIgnoreCase(type)) {
            if (contact.getBusinessDetail() == null) return;
            phonesList = contact.getBusinessDetail().getPhones();
            bigType = 2;
        } else
            return;

        if (phonesList == null) phonesList = new LinkedList();

        Phone phone = new Phone();
        phone.setPhoneType(type);
        phone.setPropertyValue(newPhone.toString());

        phonesList.add(phone);
        if (bigType == 1) contact.getPersonalDetail().setPhones(phonesList);
        if (bigType == 2) contact.getBusinessDetail().setPhones(phonesList);
    }

    public static String removeQuotedEncoding(String vCard) {
        int index;

        String NOTE_ENCODING_QUOTED_PRINTABLE = "NOTE;ENCODING=QUOTED-PRINTABLE:";
        String NOTE_ENCODING_QUOTED_PRINTABLE_CHARSET_UTF8 = "NOTE;ENCODING=QUOTED-PRINTABLE;CHARSET=UTF-8:";
        String NOTE = "NOTE:";
        String END_VCARD = "END:VCARD";

        HashMap codes = new HashMap();
        codes.put("=20", " ");
        codes.put("=0D=0A", " ");
        codes.put("=\n", "");
        codes.put(NOTE_ENCODING_QUOTED_PRINTABLE, NOTE);
        codes.put(NOTE_ENCODING_QUOTED_PRINTABLE_CHARSET_UTF8, NOTE);

        index = vCard.indexOf(NOTE_ENCODING_QUOTED_PRINTABLE_CHARSET_UTF8);
        if (index == -1) index = vCard.indexOf(NOTE_ENCODING_QUOTED_PRINTABLE);

        if (index != -1) {
            String notes = vCard.substring(index, vCard.indexOf(END_VCARD));
            String element;
            Set keySet = codes.keySet();
            Iterator i = keySet.iterator();
            while (i.hasNext()) {
                element = (String) i.next();
                notes = notes.replaceAll(element, (String) codes.get(element));
            }
            return vCard.subSequence(0, index) + notes + END_VCARD;
        }

        return vCard;
    }

}
