package zimbra.soap;


import zimbra.soap.util.AccountInfoDetails;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.HashMap;


public class AccountInfoResponseHandler extends ResponseHandler
{

    /**
     * private member that a given time in the XML parsing
     * will contain the name of the last element found
     */
    private String crtElement;
    /**
     * private member that a given time in the XML parsing
     * will contain the value of the last element found
     */
    private String crtValue;

    /**
     * HashMap used to construct an AccountInfoResponse
     * when the getAccountInfoResponse is called
     */
    private AccountInfoDetails accountInfoMembers = new AccountInfoDetails();

    /**
     * variable showing if the start GetAccountInfoResponse element
     * has been found
     */
    private boolean foundGetAccountInfoResponseStart;
    /**
     * variable showing if the end GetAccountInfoResponse element
     * has been reached
     */
    private boolean foundGetAccountInfoResponseEnd;

    /**
     * default constructor
     */
    public AccountInfoResponseHandler()
    {

    }

    /**
     * method called by the xml parser when the begining
     * of the XML document is found
     *
     * @throws SAXException
     */

    public void startDocument()
            throws SAXException
    {
        this.foundGetAccountInfoResponseStart = false;
        this.foundGetAccountInfoResponseEnd   = false;
        this.accountInfoMembers.clear();
    }

    /**
     * method called by the xml parser when the end
     * of the XML document is found
     * @throws SAXException
     */
    public void endDocument()
            throws SAXException
    {

        //if all important elements of the authentication response have been found,
        //is ok
        if (this.accountInfoMembers.containsKey(AccountInfoDetails.K_NAME) &&
                this.accountInfoMembers.containsKey(AccountInfoDetails.K_ZIMBRA_ID) &&
                this.accountInfoMembers.containsKey(AccountInfoDetails.K_ZIMBRA_MAIL_HOST))
        {
            return;
        }
        throw new SAXException("Error in GetAccountInfoResponse!");
    }

    /**
     * handler of the event that occurs when the xml parser
     * found a new element in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @param attrs
     * @throws SAXException
     */
    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException
    {

        if(this.foundGetAccountInfoResponseEnd) return;

        //element's name
        String eName = localName;
        if("".equals(eName)) eName = qName;
        if("GetAccountInfoResponse".equals(eName)) {
            this.foundGetAccountInfoResponseStart = true;
            return;
        }
        //if the start of GetAccountInfoRespose hasn't been found
        //element is not important
        if (!this.foundGetAccountInfoResponseStart) return;

        if ("name".equals(eName)) this.crtElement = eName;

        if ("attr".equals(eName)) this.crtElement = attrs.getValue("name");
    }

    /**
     * handler of the event that occurs when the xml parser
     * founds the end of element's declaration in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String uri,
                           String localName,
                           String qName)
            throws SAXException {

        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;

        if ("GetAccountInfoResponse".equals(eName))
        {
            this.foundGetAccountInfoResponseEnd = true;
            return;
        }
        if ("name".equals(eName))
        {
            this.accountInfoMembers.put(AccountInfoDetails.K_NAME, crtValue);
            return;
        }
        if ("attr".equals(eName))
        {
            if(this.crtElement.equals("zimbraId"))
            {
                this.accountInfoMembers.put(AccountInfoDetails.K_ZIMBRA_ID, crtValue);
                return;
            }
            if(this.crtElement.equals("zimbraMailHost"))
            {
                this.accountInfoMembers.put(AccountInfoDetails.K_ZIMBRA_MAIL_HOST, crtValue);
                //return;
            }
        }

    }

    /**
     * handler of the event that occurs when character are found
     * inx the xml document
     *
     * @param charBuff
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char[] charBuff, int start, int length)
            throws SAXException
    {
        if(this.foundGetAccountInfoResponseEnd) return;
        if ("name".equals(crtElement) ||
            "zimbraId".equals(crtElement) ||
            "zimbraMailHost".equals(crtElement))
        {
            this.crtValue = new String(charBuff, start, length);
        }
    }

    public Object[] getOutcome()
    {
        return new HashMap[]{this.accountInfoMembers};
    }

}
