package zimbra.soap;

import zimbra.soap.util.ContactDetails;

import java.util.HashMap;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Iterator;

/**
 * parse handler for a ContactActionResponse response on SOAP protocol
 */
public class ContactActionResponseHandler extends ResponseHandler
{
    private LinkedList actionedIds = new LinkedList();
    private String changeToken = null;

    private boolean foundContactActionResponseStart;
    private boolean foundContactActionResponseEnd;

    public ContactActionResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundContactActionResponseStart = false;
           this.foundContactActionResponseEnd   = false;
           this.actionedIds.clear();
           this.changeToken = null;
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundContactActionResponseStart ||
              !this.foundContactActionResponseEnd)
               throw new SAXException("Broken ContactActionResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundContactActionResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("change".equals(eName)){
               this.changeToken = attrs.getValue("token");
           }

           if("ContactActionResponse".equals(eName)) {
               this.foundContactActionResponseStart = true;
               return;
           }
           //if the start of GetContactsRespose hasn't been found
           //element is not important
           if (!this.foundContactActionResponseStart) return;

           if ("action".equals(eName))
           {
               StringTokenizer st = new StringTokenizer(attrs.getValue("id"),",");
               while(st.hasMoreElements()) this.actionedIds.add(st.nextElement());
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundContactActionResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("ContactActionResponse".equals(eName))
           {
               this.foundContactActionResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           HashMap[] cds = new HashMap[this.actionedIds.size()];
           Iterator it = this.actionedIds.iterator();
           int i=0;
           while(it.hasNext())
           {
               cds[i] = new HashMap();
               cds[i].put(ContactDetails.K_ID, it.next());
               cds[i].put(ContactDetails.K_CHAGE_TOKEN, this.changeToken);
               i++;
           }
           return cds;
       }

}
