package zimbra.soap;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * parse handler for a CreateCalendarResponse response on SOAP protocol
 */
public class CreateAppointmentResponseHandler extends ResponseHandler
{
    private String calendarId;

    private boolean foundCreateCalendarResponseStart;
    private boolean foundCreateCalendarResponseEnd;

    public CreateAppointmentResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundCreateCalendarResponseStart = false;
           this.foundCreateCalendarResponseEnd   = false;
           this.calendarId = null;
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundCreateCalendarResponseStart ||
              !this.foundCreateCalendarResponseEnd ||
              this.calendarId == null
             )
               throw new SAXException("Broken CreateCalendarResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundCreateCalendarResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("CreateAppointmentResponse".equals(eName)) {
               this.foundCreateCalendarResponseStart = true;
               this.calendarId = attrs.getValue("apptId");
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundCreateCalendarResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("CreateAppointmentResponse".equals(eName))
           {
               this.foundCreateCalendarResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           return new String[]{calendarId};
       }

}
