package zimbra.soap;

import java.util.LinkedList;
import java.util.StringTokenizer;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * parse handler for a SyncResponse response on SOAP protocol that will only
 * take into consideration the appointements
 */
public class GetAllCalendarsIdsResponseHandler extends ResponseHandler
{
    private LinkedList calendars = new LinkedList();

    private boolean foundSyncResponseStart;
    private boolean foundSyncResponseEnd;

    public GetAllCalendarsIdsResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundSyncResponseStart = false;
           this.foundSyncResponseEnd   = false;
           this.calendars.clear();
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundSyncResponseStart ||
              !this.foundSyncResponseEnd)
               throw new SAXException("Broken SyncResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundSyncResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("SyncResponse".equals(eName)) {
               this.foundSyncResponseStart = true;
               return;
           }
           //if the start of SyncRespose hasn't been found
           //element is not important
           if (!this.foundSyncResponseStart) return;

           if ("appt".equals(eName))
           {
               StringTokenizer st = new StringTokenizer(attrs.getValue("ids"),",");
               while(st.hasMoreElements()) this.calendars.add(st.nextElement());

           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundSyncResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("SyncResponse".equals(eName))
           {
               this.foundSyncResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           return this.calendars.toArray();
       }

}
