package zimbra.soap;

import zimbra.soap.util.CalendarDetails;
import zimbra.soap.util.RecurrenceDetails;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import java.util.HashMap;
import java.util.List;
import java.util.LinkedList;
import java.util.logging.Logger;
import sync4j.zimbra.util.Def;


// We need the ZimbraWrapper class to acces javaToZimbraTimeZone() method
import sync4j.zimbra.util.ZimbraWrapper;

public class GetAppointementResponseHandler
    extends ResponseHandler
{

    private boolean foundGetAppointmentResponseStart = false;
    private boolean foundGetAppointmentResponseEnd   = false;
    private CalendarDetails calendarDetails          = null;
    private RecurenceHandler recurenceHandler        = new RecurenceHandler();
    private List attendees = null;

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    public GetAppointementResponseHandler()
    {

    }

    public void startDocument()
            throws SAXException
    {
        this.foundGetAppointmentResponseStart = false;
        this.foundGetAppointmentResponseEnd   = false;
        this.calendarDetails = new CalendarDetails();
        this.attendees = new LinkedList();
    }

    public void endDocument()
            throws SAXException
    {
        if(!this.foundGetAppointmentResponseStart ||
           !this.foundGetAppointmentResponseEnd)
        throw new SAXException("Broken XML!");
    }

    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException
    {

        if(this.foundGetAppointmentResponseEnd) return;

        String eName = localName;
        if("".equals(eName)) eName = qName;

        if("GetAppointmentResponse".equals(eName))
        {
            this.foundGetAppointmentResponseStart = true;
            return;
        }

        if("appt".equals(eName)){
            this.calendarDetails.put(
                    CalendarDetails.K_ID,
                    attrs.getValue("id")
            );
            return;
        }

        if("inv".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_DEFUALT_INV_ID)!=null)
                return;
            this.calendarDetails.put(
                    CalendarDetails.K_DEFUALT_INV_ID,
                    attrs.getValue("id"));
            this.calendarDetails.put(
                    CalendarDetails.K_COMP_NUM,
                    attrs.getValue("compNum"));
            return;
        }

        if("comp".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_NAME)!=null) return;
            this.calendarDetails.put(
                    CalendarDetails.K_LOCATION,
                    attrs.getValue(CalendarDetails.K_LOCATION)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_NAME,
                    attrs.getValue(CalendarDetails.K_NAME)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_STATUS,
                    attrs.getValue(CalendarDetails.K_STATUS)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_FREE_BUSSY,
                    attrs.getValue(CalendarDetails.K_FREE_BUSSY)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_ACTUAL_FREE_BUSSY,
                    attrs.getValue("fba")
            );
            this.calendarDetails.put(
                    CalendarDetails.K_TRANSPARENCY,
                    attrs.getValue(CalendarDetails.K_TRANSPARENCY)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_ALL_DAY,
                    attrs.getValue(CalendarDetails.K_ALL_DAY)
            );
            this.calendarDetails.put(
                    CalendarDetails.K_TYPE,
                    attrs.getValue(CalendarDetails.K_TYPE)
            );
            return;
        }
        if("s".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_START_DATETIME)!=null)
                return;

	    ZimbraWrapper zw = new ZimbraWrapper();

            this.calendarDetails.put(
                CalendarDetails.K_START_DATETIME,
                attrs.getValue("d")
            );
            this.calendarDetails.put(
                CalendarDetails.K_TIMEZONE,
                zw.javaToZimbraTimeZone(attrs.getValue("tz"))
            );
            return;
        }

        if("e".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_END_DATETIME)!=null)
                return;
            this.calendarDetails.put(
                CalendarDetails.K_END_DATETIME,
                attrs.getValue("d")
            );
            return;
        }

        if("or".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_ORGANIZER)!=null)
                return;
            HashMap organizer = new HashMap();
            organizer.put(CalendarDetails.K_ADDRESS, attrs.getValue("url"));
            organizer.put(CalendarDetails.K_FRIENDLY_NAME, attrs.getValue("d"));
            this.calendarDetails.put(
                CalendarDetails.K_ORGANIZER,
                organizer);
            return;
        }

        if("at".equals(eName))
        {
            HashMap attendee = new HashMap();
            attendee.put(CalendarDetails.K_ADDRESS, attrs.getValue("url"));
            attendee.put(CalendarDetails.K_FRIENDLY_NAME, attrs.getValue("d"));
            attendee.put(CalendarDetails.K_ATTENDEE_ROLE, attrs.getValue("role"));
            attendee.put(CalendarDetails.K_ATTENDEE_PTST, attrs.getValue("ptst"));
            this.attendees.add(attendee);
            return;
        }

        this.recurenceHandler.startElement(uri, localName, qName, attrs);

    }

    public void endElement(String uri,
                           String localName,
                           String qName)
        throws SAXException
    {
        if(this.foundGetAppointmentResponseEnd) return;

        String eName = localName;
        if("".equals(eName)) eName = qName;

        if("GetAppointmentResponse".equals(eName))
        {
            this.foundGetAppointmentResponseEnd = true;
            return;
        }

        if("inv".equals(eName))
        {
            if(this.calendarDetails.get(CalendarDetails.K_ATTENDEES)!=null)
                return;
            this.calendarDetails.put(CalendarDetails.K_ATTENDEES,
                                     this.attendees.toArray(new HashMap[0]));
            return;
        }

        this.recurenceHandler.endElement(uri, localName, qName);
    }

    public void characters(char [] charBuff, int start, int length)
    throws SAXException
    {
    }

    public Object[] getOutcome()
    {
        RecurrenceDetails[] recurrenceDetailses = null;
        Object[] objs = this.recurenceHandler.getOutcome();
        if(objs != null &&
           objs instanceof RecurrenceDetails[])
        {
            recurrenceDetailses = (RecurrenceDetails[]) objs;
            for(int i=0; i<recurrenceDetailses.length; i++)
            {
                if(recurrenceDetailses[i].get(RecurrenceDetails.K_DATETIME_START)==null)
                    recurrenceDetailses[i].put(RecurrenceDetails.K_DATETIME_START,
                            calendarDetails.get(CalendarDetails.K_START_DATETIME));
                if(recurrenceDetailses[i].get(RecurrenceDetails.K_TIMEZONE)==null)
                    recurrenceDetailses[i].put(RecurrenceDetails.K_TIMEZONE,
                            calendarDetails.get(CalendarDetails.K_TIMEZONE));
            }
        }
        this.calendarDetails.put(CalendarDetails.K_RECURRENCE,
                                 recurrenceDetailses);

        return new CalendarDetails[]{calendarDetails};
    }


}
