package zimbra.soap;

import zimbra.soap.util.ContactDetails;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.logging.Logger;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import sync4j.zimbra.util.Def;

/**
 * parse handler for a GetContactsResponse response on SOAP protocol
 */
public class GetContactsResponseHandler extends ResponseHandler {
    private Object[] usedDetails;

    private LinkedList contacts = new LinkedList();

    private ContactDetails crtContactDetails;

    private String crtElement;
    private String lastElement = "";

    private String crtValue;

    private boolean foundGetContactsResponseStart;
    private boolean foundGetContactsResponseEnd;

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    public GetContactsResponseHandler() {
        this.usedDetails = ContactDetails.keysArray;
    }

    public GetContactsResponseHandler(Object[] usedDetails) {
        this.usedDetails = usedDetails;
    }

    /**
     * method called by the xml parser when the begining
     * of the XML document is found
     *
     * @throws org.xml.sax.SAXException
     */

    public void startDocument()
            throws SAXException {
        this.foundGetContactsResponseStart = false;
        this.foundGetContactsResponseEnd = false;
        this.contacts.clear();
    }

    /**
     * method called by the xml parser when the end
     * of the XML document is found
     *
     * @throws SAXException
     */
    public void endDocument()
            throws SAXException {
        if (!this.foundGetContactsResponseEnd)
            throw new SAXException("Broken GetContactsResponse!");
    }

    /**
     * handler of the event that occurs when the xml parser
     * found a new element in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @param attrs
     * @throws SAXException
     */
    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException {

        if (this.foundGetContactsResponseEnd) return;

        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;

        if ("GetContactsResponse".equals(eName)) {
            this.foundGetContactsResponseStart = true;
            return;
        }
        //if the start of GetContactsRespose hasn't been found
        //element is not important
        if (!this.foundGetContactsResponseStart) return;

        if ("cn".equals(eName)) {
            this.crtElement = null;
            this.crtContactDetails = new ContactDetails();
            this.crtContactDetails.put(ContactDetails.K_ID,
                    attrs.getValue(ContactDetails.K_ID));
            this.crtContactDetails.put(ContactDetails.K_FOLDER,
                    attrs.getValue(ContactDetails.K_FOLDER));
            this.crtContactDetails.put(ContactDetails.K_CHAGE_TOKEN,
                    attrs.getValue(ContactDetails.K_CHAGE_TOKEN));
            return;
        }

        if ("a".equals(eName) && this.isUsedDetail(attrs.getValue("n"))) {
            this.crtElement = attrs.getValue("n");
            return;
        }
        this.crtElement = null;
    }

    /**
     * handler of the event that occurs when the xml parser
     * founds the end of element's declaration in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String uri,
                           String localName,
                           String qName)
            throws SAXException {

        if (!this.foundGetContactsResponseStart) return;
        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;

        if ("GetContactsResponse".equals(eName)) {
            this.foundGetContactsResponseEnd = true;
            return;
        }
        if ("cn".equals(eName)) {
            this.contacts.add(this.crtContactDetails);
            return;
        }
        if ("a".equals(eName) && this.crtElement != null) {
            this.crtContactDetails.put(this.crtElement, this.crtValue);
        }

    }

    /**
     * handler of the event that occurs when character are found
     * inx the xml document
     *
     * @param charBuff
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char[] charBuff, int start, int length)
            throws SAXException {
        if (this.foundGetContactsResponseEnd) return;
        if (this.crtElement != null) {
            String value = new String(charBuff, start, length);

            if (this.crtElement.equals(ContactDetails.K_NOTES)) {
                if (lastElement.equals(ContactDetails.K_NOTES))
                    this.crtValue += value;
                else
                    this.crtValue = value;
            } else
                this.crtValue = value;

            this.lastElement = this.crtElement;
        }
    }

    public Object[] getOutcome() {
        HashMap[] cds = new HashMap[this.contacts.size()];
        Iterator it = this.contacts.iterator();
        int i = 0;
        while (it.hasNext()) {
            cds[i] = (ContactDetails) it.next();
            i++;
        }
        return cds;
    }

    private boolean isUsedDetail(String det) {
        for (int i = 0; i < this.usedDetails.length; i++) {
            if (det.equals(this.usedDetails[i])) return true;
        }
        return false;
    }


}
