package zimbra.soap;

import zimbra.soap.util.ItemDetails;

import java.util.HashMap;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Iterator;

/**
 * parse handler for a CreateItemResponse response on SOAP protocol
 */
public class ItemActionResponseHandler extends ResponseHandler
{
    private LinkedList actionedIds = new LinkedList();

    private boolean foundItemActionResponseStart;
    private boolean foundItemActionResponseEnd;

    public ItemActionResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundItemActionResponseStart = false;
           this.foundItemActionResponseEnd   = false;
           this.actionedIds.clear();
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundItemActionResponseStart ||
              !this.foundItemActionResponseEnd)
               throw new SAXException("Broken ItemActionResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundItemActionResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("ItemActionResponse".equals(eName)) {
               this.foundItemActionResponseStart = true;
               return;
           }
           //if the start of GetItemsRespose hasn't been found
           //element is not important
           if (!this.foundItemActionResponseStart) return;

           if ("action".equals(eName))
           {
               StringTokenizer st = new StringTokenizer(attrs.getValue("id"),",");
               while(st.hasMoreElements()) this.actionedIds.add(st.nextElement());
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundItemActionResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("ItemActionResponse".equals(eName))
           {
               this.foundItemActionResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           HashMap[] cds = new HashMap[this.actionedIds.size()];
           Iterator it = this.actionedIds.iterator();
           int i=0;
           while(it.hasNext())
           {
               cds[i] = new HashMap();
               cds[i].put(ItemDetails.K_ID, it.next());
               i++;
           }
           return cds;
       }

}
