package zimbra.soap;

import zimbra.soap.util.AuthDetails;
import zimbra.soap.util.CalendarDetails;
import zimbra.soap.response.AuthResponse;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.HashMap;
import java.util.logging.Logger;

import sync4j.zimbra.util.Def;

/**
 * parse handler for a AuthResponse response on SOAP protocol
 */
public class MessageResponseHandler extends ResponseHandler {

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    public MessageResponseHandler() {

    }

    private boolean isStartDate = false;
    private boolean isEndDate = false;

    private CalendarDetails calendarDetails = new CalendarDetails();

    /**
     * method called by the xml parser when the begining
     * of the XML document is found
     *
     * @throws SAXException
     */

    public void startDocument()
            throws SAXException {
    }

    /**
     * method called by the xml parser when the end
     * of the XML document is found
     *
     * @throws SAXException
     */
    public void endDocument()
            throws SAXException {

        //if all important elements of the authentication response have been found,
        //is ok
    }

    /**
     * handler of the event that occurs when the xml parser
     * found a new element in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @param attrs
     * @throws SAXException
     */
    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException {

        String element = qName;
        if (element == null) element = localName;

        if (localName.equalsIgnoreCase("s") || qName.equalsIgnoreCase("s")) isStartDate = true;
        if (localName.equalsIgnoreCase("e") || qName.equalsIgnoreCase("e")) isEndDate = true;

        if ((isStartDate || isEndDate) && attrs.getLength() > 0) {
            for (int i = 0; i < attrs.getLength(); i++)
                if (attrs.getQName(i).equalsIgnoreCase("tz")) {
                    calendarDetails.put(CalendarDetails.K_TIMEZONE, attrs.getValue(i));
                } else if (attrs.getQName(i).equalsIgnoreCase("d") && isStartDate) {
                    log.finest("Start date is: " + attrs.getValue(i));
                    calendarDetails.put(CalendarDetails.K_START_DATETIME, attrs.getValue(i));
                } else if (attrs.getQName(i).equalsIgnoreCase("d")) {
                    log.finest("End date is: " + attrs.getValue(i));
                    calendarDetails.put(CalendarDetails.K_END_DATETIME, attrs.getValue(i));
                }
        }
    }

    /**
     * handler of the event that occurs when the xml parser
     * founds the end of element's declaration in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String uri,
                           String localName,
                           String qName)
            throws SAXException {

        if (localName.equalsIgnoreCase("s") || qName.equalsIgnoreCase("s")) isStartDate = false;
        if (localName.equalsIgnoreCase("e") || qName.equalsIgnoreCase("e")) isEndDate = false;

    }

    /**
     * handler of the event that occurs when character are found
     * inx the xml document
     *
     * @param charBuff
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char[] charBuff, int start, int length)
            throws SAXException {
    }

    /**
     * @return a new AuthResonse object with the propreties obtained
     *         by parsing the xml document
     */
    public Object[] getOutcome() {
        return new HashMap[]{this.calendarDetails};
    }

}
