package zimbra.soap;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * parse handler for a ModifyAppointmentResponse response on SOAP protocol
 */
public class ModifyAppointmentResponseHandler extends ResponseHandler
{
    private String invId;

    private boolean foundModifyAppointmentResponseStart;
    private boolean foundModifyAppointmentResponseEnd;

    public ModifyAppointmentResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundModifyAppointmentResponseStart = false;
           this.foundModifyAppointmentResponseEnd   = false;
           this.invId = null;
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundModifyAppointmentResponseStart ||
              !this.foundModifyAppointmentResponseEnd ||
              this.invId == null
             )
               throw new SAXException("Broken ModifyAppointmentResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundModifyAppointmentResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("ModifyAppointmentResponse".equals(eName)) {
               this.foundModifyAppointmentResponseStart = true;
               this.invId = attrs.getValue("invId").split("-")[1];
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundModifyAppointmentResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("ModifyAppointmentResponse".equals(eName))
           {
               this.foundModifyAppointmentResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           return new String[]{invId};
       }

}
