package zimbra.soap;

import zimbra.soap.util.ContactDetails;

import java.util.HashMap;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * parse handler for a CreateContactResponse response on SOAP protocol
 */
public class ModifyContactResponseHandler extends ResponseHandler
{
    private ContactDetails contactDetails;

    private boolean foundModifyContactResponseStart;
    private boolean foundModifyContactResponseEnd;

    public ModifyContactResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this. foundModifyContactResponseStart = false;
           this. foundModifyContactResponseEnd   = false;
           this.contactDetails = new ContactDetails();
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundModifyContactResponseStart ||
              !this.foundModifyContactResponseEnd ||
              !this.contactDetails.containsKey(ContactDetails.K_ID)
             )
               throw new SAXException("Broken CreateContactResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this. foundModifyContactResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("ModifyContactResponse".equals(eName)) {
               this. foundModifyContactResponseStart = true;
               return;
           }
           //if the start of CreateContactRespose hasn't been found
           //element is not important
           if (!this. foundModifyContactResponseStart) return;

           if ("cn".equals(eName))
           {
               this.contactDetails.put(ContactDetails.K_ID,
                                        attrs.getValue(ContactDetails.K_ID));
               this.contactDetails.put(ContactDetails.K_FOLDER,
                                        attrs.getValue(ContactDetails.K_FOLDER));
               this.contactDetails.put(ContactDetails.K_CHAGE_TOKEN,
                                        attrs.getValue(ContactDetails.K_CHAGE_TOKEN));
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this. foundModifyContactResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("ModifyContactResponse".equals(eName))
           {
               this. foundModifyContactResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           return new HashMap[]{contactDetails};
       }

}
