package zimbra.soap;

import zimbra.soap.util.RecurrenceDetails;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.List;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.HashMap;

// We need the ZimbraWrapper class to acces javaToZimbraTimeZone() method
import sync4j.zimbra.util.ZimbraWrapper;


// TODO: This routine parses the <recur> block from Zimbra, but doesn't handle all
// of the parameters that the server can pass properly. All paramaters need to be dealt with
// to accurately parse an event. Specific parameters that are being fudged:
// <dates>,<dtval> blocks -- can't handle more than one date
// <except>,<cancel> blocks  -- ignored completely

public class RecurenceHandler extends ResponseHandler
{
    private List recurences = null;
    private RecurrenceDetails recurrenceDetails;
    private List byDayDetails;

    public RecurenceHandler()
    {
        this.recurences = new LinkedList();
    }

    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
    throws SAXException
    {
        String eName = localName;
        if("".equals(eName)) eName = qName;

        if("recur".equals(eName))
        {
            recurrenceDetails = new RecurrenceDetails();
            return;
        }

        if("add".equals(eName) || "exclude".equals(eName))
        {
            recurences.clear();
            recurrenceDetails.put(RecurrenceDetails.K_ADD_EXCLUDE, eName);

	    // TODO: This is just plain wrong, but we're setting the INTERVAL to 1 by default
	    // because code elsewhere throws an exception if no INTERVAL is set. An 'exclude'
	    // can have a list of dates that don't have repeating intervals, so it IS correct
	    // to have a <recur> block that doesn't have an INTERVAL set.
	
            recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL, "1");
            return;
        }

        if("date".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_DATETIME_START,
                                 attrs.getValue("d"));
            this.recurrenceDetails.put(RecurrenceDetails.K_TIMEZONE,
                                      attrs.getValue("tz"));
            return;
        }

        if("rule".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                                 attrs.getValue("freq"));
            return;
        }

        if("until".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_DATETIME_END,
                                 attrs.getValue("d"));
            return;
        }

        if("count".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_COUNT,
                                 attrs.getValue("num"));
            return;
        }

        if("interval".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                                 attrs.getValue("ival"));
            return;
        }

        if("bysecond".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("seclist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYSECOND,
                                 list.toArray(new String[0]));
            return;
        }

        if("byminute".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("minlist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMINUTE,
                                 list.toArray(new String[0]));
            return;
        }

        if("byhour".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("hrlist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYHOUR,
                                 list.toArray(new String[0]));
            return;
        }

        if("byday".equals(eName))
        {
            this.byDayDetails = new LinkedList();
            return;
        }

        if("wkday".equals(eName))
        {
            HashMap wkday = new HashMap();
            wkday.put(RecurrenceDetails.K_RULE_BYDAY_ORDWK, attrs.getValue("ordwk"));
            wkday.put(RecurrenceDetails.K_RULE_BYDAY_DAY,   attrs.getValue("day"));
            this.byDayDetails.add(wkday);
            return;
        }

        if("bymonthday".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("modaylist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTHDAY,
                                 list.toArray(new String[0]));
            return;
        }

        if("byyearday".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("yrdaylist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYYEARDAY,
                                 list.toArray(new String[0]));
            return;
        }

        if("byweekno".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("wklist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYWEEKNO,
                                 list.toArray(new String[0]));
            return;
        }

        if("bymonth".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("molist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTH,
                                 list.toArray(new String[0]));
            return;
        }

        if("bysetpos".equals(eName))
        {
            List list = new LinkedList();
            StringTokenizer st = new StringTokenizer(attrs.getValue("poslist"), ",");
            while(st.hasMoreElements()) list.add(st.nextToken());
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYSETPOS,
                                 list.toArray(new String[0]));
            return;
        }

        if("wkst".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_WKST,
                                 attrs.getValue("day"));
        }

	if("dates".equals(eName))
	{
	    // TODO: This should set up an array, as 'dates' can have multiple values
	    // Figure out how to pass such an array back.

            ZimbraWrapper zw = new ZimbraWrapper();

            this.recurrenceDetails.put(RecurrenceDetails.K_TIMEZONE,
                zw.javaToZimbraTimeZone(attrs.getValue("tz")));
		//                      attrs.getValue("tz"));
	    return;
	}

	if("dtval".equals(eName))
	{
	    // TODO: 'dtval' can't appear outside of a 'dates' block, but doesn't actually
	    // carry any of its own parameters - ignore??
	    return;
	}

        if("s".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_DATETIME_START,
                                 attrs.getValue("d"));
            return;
	}

        if("e".equals(eName))
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_DATETIME_END,
                                 attrs.getValue("d"));
            return;
        }

	if("dur".equals(eName))
	{
	    // TODO: 'dur' goes inside a 'dtval' block, and replaced 'e', but
	    // what's the syntax?? Treat it as an 'e' for now (WRONG THING TO DO!!)

            recurrenceDetails.put(RecurrenceDetails.K_RULE_DATETIME_END,
                                 attrs.getValue("d"));
	    return;
	}

    }

    public void endElement(String uri,
                           String localName,
                           String qName)
    throws SAXException
    {
        String eName = localName;
        if("".equals(eName)) eName = qName;

        if("add".equals(eName) || "exclude".equals(eName))
        {
            this.recurences.add(this.recurrenceDetails);
            return;
        }

        if("wkday".equals(eName))
        {
            this.recurrenceDetails.put(RecurrenceDetails.K_RULE_BYDAY,
                                      this.byDayDetails.toArray(new HashMap[0]));
        }
    }

    public Object[] getOutcome()
    {
        return recurences.toArray(new RecurrenceDetails[0]);
    }
}
