package zimbra.soap;

import zimbra.soap.util.ContactDetails;
import zimbra.soap.util.SyncDetails;
import zimbra.soap.util.CalendarDetails;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.StringTokenizer;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

/**
 * parse handler for a SyncResponse response on SOAP protocol
 */
public class SyncResponseHandler extends ResponseHandler
{
    private LinkedList deleted = new LinkedList();
    private LinkedList contacts = new LinkedList();
    private LinkedList calendars = new LinkedList();
    private String modifiedDate;
    private String token;

    private boolean foundSyncResponseStart;
    private boolean foundSyncResponseEnd;

    public SyncResponseHandler()
    {
    }


    /**
        * method called by the xml parser when the begining
        * of the XML document is found
        *
        * @throws org.xml.sax.SAXException
        */

       public void startDocument()
               throws SAXException
       {
           this.foundSyncResponseStart = false;
           this.foundSyncResponseEnd   = false;
           this.deleted.clear();
           this.contacts.clear();
           this.calendars.clear();
           this.modifiedDate = null;
           this.token        = null;
       }

       /**
        * method called by the xml parser when the end
        * of the XML document is found
        * @throws org.xml.sax.SAXException
        */
       public void endDocument()
               throws SAXException
       {
           if(!this.foundSyncResponseStart ||
              !this.foundSyncResponseEnd)
               throw new SAXException("Broken SyncResponse!");
       }

       /**
        * handler of the event that occurs when the xml parser
        * found a new element in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @param attrs
        * @throws org.xml.sax.SAXException
        */
       public void startElement(String uri,
                                String localName,
                                String qName,
                                Attributes attrs)
               throws SAXException
       {

           if(this.foundSyncResponseEnd) return;

           //element's name
           String eName = localName;
           if("".equals(eName)) eName = qName;

           if("SyncResponse".equals(eName)) {
               this.foundSyncResponseStart = true;
               this.token        = attrs.getValue(SyncDetails.K_TOKEN);
               this.modifiedDate = attrs.getValue(SyncDetails.K_MODIFIED_DATE);
               return;
           }
           //if the start of SyncRespose hasn't been found
           //element is not important
           if (!this.foundSyncResponseStart) return;

           if ("deleted".equals(eName))
           {
               StringTokenizer st = new StringTokenizer(attrs.getValue("ids"),",");
               while(st.hasMoreElements()) this.deleted.add(st.nextElement());

           }

           if ("cn".equals(eName))
           {
               HashMap contact = new HashMap();
               contact.put(ContactDetails.K_ID,
                           attrs.getValue("id"));
               contact.put(ContactDetails.K_MODIFICATION_DATE,
                           attrs.getValue("md"));
               contact.put(ContactDetails.K_FOLDER,
                           attrs.getValue("l"));
               this.contacts.add(contact);

           }

           if ("appt".equals(eName))
           {
               HashMap calendar = new HashMap();
               calendar.put(CalendarDetails.K_ID,
                           attrs.getValue("id"));
               calendar.put(CalendarDetails.K_MODIFICATION_DATE,
                           attrs.getValue("md"));
               this.calendars.add(calendar);
           }
       }

       /**
        * handler of the event that occurs when the xml parser
        * founds the end of element's declaration in the XML document
        *
        * @param uri
        * @param localName
        * @param qName
        * @throws org.xml.sax.SAXException
        */
       public void endElement(String uri,
                              String localName,
                              String qName)
               throws SAXException
       {

           if(!this.foundSyncResponseStart) return;
           //element's name
           String eName = localName;
           if ("".equals(eName)) eName = qName;

           if ("SyncResponse".equals(eName))
           {
               this.foundSyncResponseEnd = true;
           }
       }


       public Object[] getOutcome()
       {
           SyncDetails ad = new SyncDetails();
           ad.put(SyncDetails.K_MODIFIED_DATE, this.modifiedDate);
           ad.put(SyncDetails.K_TOKEN, this.token);
           ad.put(SyncDetails.K_DELETED, this.deleted.toArray());
           ad.put(SyncDetails.K_CONTACTS, this.contacts.toArray());
           ad.put(SyncDetails.K_CALENDARS, this.calendars.toArray());
           return new HashMap[]{ad};
       }

}
