package zimbra.soap.util;

import zimbra.soap.exceptions.MalformedRRuleException;

import java.util.List;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.HashMap;

public class RRuleParser
{
    public static final String FREQUENCY_DAILY              = "D";
    public static final String FREQUENCY_WEEKLY             = "W";
    public static final String FREQUENCY_MONTHLY_BY_POSITION = "MP";
    public static final String FREQUENCY_MONTHLY_BY_DAY     = "MD";
    public static final String FREQUENCY_YEARLY_BY_MONTH    = "YM";
    public static final String FREQUENCY_YEARLY_BY_DAY      = "YD";

    public static final String SUNDAY    = "SU";
    public static final String MONDAY    = "MO";
    public static final String TUESDAY   = "TU";
    public static final String WEDNESDAY = "WE";
    public static final String THURSDAY  = "TH";
    public static final String FRIDAY    = "FR";
    public static final String SATURDAY  = "SA";

    public static  final String LAST_DAY = "LD";

    int index;
    String rRule;
    RecurrenceDetails recurrenceDetails;

    RRuleParser()
    {

    }

    RRuleParser(String rRule)
    {
        this.rRule = rRule;
    }

    public void setRRule(String rRule)
    {
        this.rRule = rRule;
    }

    public RecurrenceDetails getRecurrenceDetails()
    throws MalformedRRuleException
    {
        this.index = 0;
        this.recurrenceDetails = new RecurrenceDetails();

        recurrenceDetails.put(RecurrenceDetails.K_ADD_EXCLUDE,
                             RecurrenceDetails.V_ADD);

        if(this.rRule.startsWith(FREQUENCY_DAILY)) this.getDailyRecurrence();
        else if(this.rRule.startsWith(FREQUENCY_WEEKLY)) this.getWeeklyRecurrence();
        else if(rRule.startsWith(FREQUENCY_MONTHLY_BY_POSITION)) this.getMonthlyByPosRecurrence();
        else if(rRule.startsWith(FREQUENCY_MONTHLY_BY_DAY)) this.getMonthlyByDayRecurrence();
        else if(rRule.startsWith(FREQUENCY_YEARLY_BY_MONTH)) this.getYearlyByMonthRecurrence();
        else if(rRule.startsWith(FREQUENCY_YEARLY_BY_DAY)) this.getYearlyByDayRecurrence();

        if(this.recurrenceDetails.containsKey(RecurrenceDetails.K_RULE_FREQUENCY))
        {
            this.ignoreBlanks();

            String numOccurrences = getNumOccurrences();
            if(numOccurrences!=null)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_COUNT,
                                 numOccurrences);

            this.ignoreBlanks();

            String endDate = this.getEndDate();
            if(endDate!=null)
                this.recurrenceDetails.put(RecurrenceDetails.K_RULE_DATETIME_END, endDate);
            return this.recurrenceDetails;
        }

        throw new MalformedRRuleException("Invalid frequency in recurrence rule: "
                                          +rRule);
    }

    public static RecurrenceDetails parse(String rRule)
    throws MalformedRRuleException
    {
        RRuleParser rRuleParser = new RRuleParser(rRule);
        return rRuleParser.getRecurrenceDetails();
    }

    private void getDailyRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_DAILY);

        this.index = 1;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             getInterval());
    }

    private void getWeeklyRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_WEEKLY);
        this.index = 1;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             getInterval());

        this.ignoreBlanks();

        String[] weekDays = this.getWeekDayList();
        if(weekDays!=null)
        {
            HashMap[] hm = new HashMap[weekDays.length];
            for(int i=0; i<hm.length; i++)
            {
                hm[i] = new HashMap();
                hm[i].put(RecurrenceDetails.K_RULE_BYDAY_DAY,
                          weekDays[i]);
            }
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYDAY, hm);

        }
    }

    private void getMonthlyByPosRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_MONTHLY);
        this.index = 2;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             this.getInterval());
        this.ignoreBlanks();
        HashMap[] occurrences = this.getOccurrencesInMonth();
        if(occurrences!= null)
        {
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYDAY,
                                 occurrences);
        }
    }

    private void getMonthlyByDayRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_MONTHLY);
        this.index = 2;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             getInterval());

        this.ignoreBlanks();

        String[] occurrences = this.getDayOccurrencesInMonth();
        if(occurrences!=null)
            this.recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTHDAY,
                                      occurrences);
    }

    private void getYearlyByMonthRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_YEARLY);
        this.index = 2;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             getInterval());
        this.ignoreBlanks();

        String[] months = this.getMonthsList();
        if(months!=null)
            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTH,
                                 months);
    }

    private void getYearlyByDayRecurrence()
    throws MalformedRRuleException
    {
        recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY,
                             RecurrenceDetails.V_RULE_FREQUENCY_YEARLY);
        index = 2;

        recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL,
                             getInterval());

        this.ignoreBlanks();

        String[] days = this.getYearDaysList();
        if(days!=null)
            this.recurrenceDetails.put(RecurrenceDetails.K_RULE_BYYEARDAY,
                                      days);
    }

    private String getInterval()
    throws MalformedRRuleException
    {
        String interval;
        try{interval = getNumericalPart(false);}
        catch(NumberFormatException e)
        {
            throw new
            MalformedRRuleException("Invalid reccurence interval in rrule: " +
                                    rRule, e);
        }

        return interval;
    }

    private String getNumOccurrences()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        String numOccurrences = null;
        if(rRule.charAt(this.index)=='#')
        {
            try
            {
                this.index++;
                numOccurrences = getNumericalPart(false);
            }
            catch(NumberFormatException e)
            {
                throw new
                MalformedRRuleException(
                        "Invalid number of occurrences in rrule: " + rRule, e);
            }
        }
        if("0".equals(numOccurrences)) return null;
        return numOccurrences;
    }

    private String[] getWeekDayList()
    throws MalformedRRuleException
    {
        List weekDayList = new LinkedList();
        try
        {
            StringTokenizer st =
            new StringTokenizer(this.rRule.substring(this.index,
                                this.rRule.length()), " ");
            int numElems = 0;
            while(st.hasMoreElements())
            {
                if(numElems>0) this.index++;
                numElems++;
                Object element = st.nextElement();
                if(SUNDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_SUNDAY);
                    this.index += SUNDAY.length();
                    continue;
                }
                if(MONDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_MONDAY);
                    this.index += MONDAY.length();
                    continue;
                }
                if(TUESDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_TUESDAY);
                    this.index += TUESDAY.length();
                    continue;
                }
                if(WEDNESDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_WEDNESDAY);
                    this.index += WEDNESDAY.length();
                    continue;
                }
                if(THURSDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_THURSDAY);
                    this.index += THURSDAY.length();
                    continue;
                }
                if(FRIDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_FRIDAY);
                    this.index += FRIDAY.length();
                    continue;
                }
                if(SUNDAY.equals(element))
                {
                    weekDayList.add(RecurrenceDetails.V_SUNDAY);
                    this.index += SUNDAY.length();
                }
                break;
            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid week days list in rrule: " +
                                    rRule, e);
        }
        if(weekDayList.size()<=0) return null;
        return (String[])weekDayList.toArray(new String[0]);
    }

    private String getEndDate()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        StringBuffer endDate=new StringBuffer();
        try
        {
            endDate.append(this.getNumericalPart(true));
            if(this.index<this.rRule.length() && this.rRule.charAt(index)=='T')
            {
                endDate.append('T');
                index++;
                endDate.append(this.getNumericalPart(true));
                if(this.index<this.rRule.length() && this.rRule.charAt(index)=='Z')
                {
                    endDate.append('Z');
                    index++;
                }
            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid end date in rrule: "+ rRule, e);
        }
        if(endDate.length()<=0) return null;
        return endDate.toString();
    }

    public String[] getDayOccurrencesInMonth()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        List occurrences = new LinkedList();
        try
        {
            StringTokenizer st = new StringTokenizer(this.rRule.substring(index), " ");
            int numElems = 0;
            while(st.hasMoreElements())
            {
                if(numElems>0) index++;
                numElems++;
                Object element = st.nextElement();
                if(LAST_DAY.equals(element))
                {
                    occurrences.add("-1");
                    index+=LAST_DAY.length();
                    continue;
                }
                int oldIndex = this.index;
                String ocurr;
                try{ocurr = this.getNumericalPart(false);}
                catch(NumberFormatException e){break;}
                if(Integer.parseInt(ocurr)<1 || Integer.parseInt(ocurr)>31)
                {
                    this.index = oldIndex;
                    break;
                }
                if(this.index<this.rRule.length() &&
                   (this.rRule.charAt(this.index)=='+' ||
                    this.rRule.charAt(this.index)=='-'))
                    occurrences.add(this.rRule.charAt(this.index++)+ocurr);
                else
                    if(this.index<this.rRule.length() &&
                       this.rRule.charAt(this.index)=='T')
                    {
                        index-=8;
                        break;
                    }
                    else occurrences.add(ocurr);
            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid occurrences list in rrule: " +
                                    this.rRule, e);
        }
        if(occurrences.size()<=0) return null;
        return (String[])occurrences.toArray(new String[0]);
    }

    public HashMap[] getOccurrencesInMonth()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        List occurrences = new LinkedList();
        try
        {
            StringTokenizer st = new StringTokenizer(this.rRule.substring(index), " ");
            int numElems = 0;
            while(st.hasMoreElements())
            {
                String occurr;
                if(numElems>0) index++;
                numElems++;
                Object element = st.nextElement();
                if(LAST_DAY.equals(element))
                {
                    index+=LAST_DAY.length();
                    occurr = "-1";
                }
                else
                {
                    int oldIndex = this.index;
                    try{occurr = this.getNumericalPart(false);}
                    catch(NumberFormatException e){break;}
                    if(Integer.parseInt(occurr)<1 || Integer.parseInt(occurr)>31)
                    {
                        this.index = oldIndex;
                        break;
                    }
                    if(this.index<this.rRule.length() &&
                       (this.rRule.charAt(this.index)=='+' ||
                        this.rRule.charAt(this.index)=='-'))
                        occurr = this.rRule.charAt(this.index++) + occurr;
                    else
                        if(this.index<this.rRule.length() &&
                        this.rRule.charAt(this.index)=='T')
                        {
                            index-=8;
                            break;
                        }
                }

                String[] weekDays = this.getWeekDayList();
                for(int i=0; i<weekDays.length; i++)
                {
                    st.nextElement();
                    HashMap hm = new HashMap();
                    hm.put(RecurrenceDetails.K_RULE_BYDAY_DAY,   weekDays[i]);
                    hm.put(RecurrenceDetails.K_RULE_BYDAY_ORDWK, occurr);
                    occurrences.add(hm);
                }

            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid occurrences list in rrule: " +
                                    this.rRule, e);
        }
        if(occurrences.size()<=0) return null;
        return (HashMap[])occurrences.toArray(new HashMap[0]);
    }

    public String[] getMonthsList()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        List months = new LinkedList();
        try
        {
            StringTokenizer st = new StringTokenizer(this.rRule.substring(index), " ");
            int numElems = 0;
            while(st.hasMoreElements())
            {
                if(numElems>0) index++;
                numElems++;
                st.nextElement();
                int oldIndex = this.index;
                String month;
                try{month = this.getNumericalPart(false);}
                catch(NumberFormatException e){break;}
                if(Integer.parseInt(month)<1 || Integer.parseInt(month)>12)
                {
                    this.index = oldIndex;
                    break;
                }
                months.add(month);
            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid months list in rrule: " +
                                    this.rRule, e);
        }
        if(months.size()<=0) return null;
        return (String[])months.toArray(new String[0]);
    }

    public String[] getYearDaysList()
    throws MalformedRRuleException
    {
        if(this.index>=this.rRule.length()) return null;
        List days = new LinkedList();
        try
        {
            StringTokenizer st = new StringTokenizer(this.rRule.substring(index), " ");
            int numElems = 0;
            while(st.hasMoreElements())
            {
                if(numElems>0) index++;
                numElems++;
                st.nextElement();
                int oldIndex = this.index;
                String day;
                try{day = this.getNumericalPart(false);}
                catch(NumberFormatException e){break;}
                if(Integer.parseInt(day)<1 || Integer.parseInt(day)>366)
                {
                    this.index = oldIndex;
                    break;
                }
                days.add(day);
            }
        }
        catch(Exception e)
        {
            throw new
            MalformedRRuleException("Invalid days list in rrule: " +
                                    this.rRule, e);
        }
        if(days.size()<=0) return null;
        return (String[])days.toArray(new String[0]);
    }

    private String getNumericalPart(boolean leadingZeros)
    throws NumberFormatException
    {
        int start = this.index;
        int end;

        while(start<this.rRule.length() && !leadingZeros &&
              this.rRule.charAt(start)=='0')
            start++;
        end = start;
        while(end<this.rRule.length() &&
              this.rRule.charAt(end)>='0' &&
              this.rRule.charAt(end)<='9')
            end++;
        if(end==start)
        {
            if(start>this.index){
                this.index = end;
                return "0";
            }
            throw new NumberFormatException(
                    "No numerical part at index " +
                    this.index +
                    " of the string " +
                    this.rRule+"!");
        }
        this.index = end;
        return this.rRule.substring(start, end);
    }

    private void ignoreBlanks()
    {
        while(index<this.rRule.length() &&
              this.rRule.charAt(this.index)==' ') this.index++;
    }
}
