package zimbra.soap.util;

import sync4j.zimbra.util.UtilityCalendar;

import java.util.HashMap;

public class RecurrenceDetails extends HashMap
{
    public static final String K_ADD_EXCLUDE       = "add_exclude";
    public static final String K_DATETIME_START    = "date";
    public static final String K_TIMEZONE          = "tz";
    public static final String K_RULE_FREQUENCY    = "freq";
    public static final String K_RULE_DATETIME_END = "until";
    public static final String K_RULE_COUNT        = "count";
    public static final String K_RULE_INTERVAL     = "interval";
    public static final String K_RULE_BYSECOND     = "bysecond";
    public static final String K_RULE_BYMINUTE     = "byminute";
    public static final String K_RULE_BYHOUR       = "byhour";
    public static final String K_RULE_BYDAY        = "byday";
    public static final String K_RULE_BYDAY_ORDWK  = "ordwk";
    public static final String K_RULE_BYDAY_DAY    = "day";
    public static final String K_RULE_BYMONTHDAY   = "bymonthday";
    public static final String K_RULE_BYYEARDAY    = "byyearday";
    public static final String K_RULE_BYWEEKNO     = "byweekno";
    public static final String K_RULE_BYMONTH      = "bymonth";
    public static final String K_RULE_BYSETPOS     = "bysetpos";
    public static final String K_RULE_WKST         = "wkst";

    protected String[] keysArray = new String[]
            {K_ADD_EXCLUDE, K_DATETIME_START, K_TIMEZONE, K_RULE_FREQUENCY,
             K_RULE_DATETIME_END, K_RULE_COUNT, K_RULE_INTERVAL, K_RULE_BYSECOND,
             K_RULE_BYMINUTE, K_RULE_BYHOUR, K_RULE_BYDAY, K_RULE_BYDAY_ORDWK,
             K_RULE_BYDAY_DAY, K_RULE_BYMONTHDAY, K_RULE_BYYEARDAY,
             K_RULE_BYWEEKNO, K_RULE_BYMONTH, K_RULE_BYSETPOS, K_RULE_WKST};

    //possible values for add/exclude
    public static final String V_ADD     = "add";
    public static final String V_EXCLUDE = "exclude";

    //possible values for recurrence rule frequency
    public static final String V_RULE_FREQUENCY_SECONDS = "SEC";
    public static final String V_RULE_FREQUENCY_MINUTES = "MIN";
    public static final String V_RULE_FREQUENCY_HOURS   = "HOU";
    public static final String V_RULE_FREQUENCY_DAILY   = "DAI";
    public static final String V_RULE_FREQUENCY_WEEKLY  = "WEE";
    public static final String V_RULE_FREQUENCY_MONTHLY = "MON";
    public static final String V_RULE_FREQUENCY_YEARLY  = "YEA";

    //possible values for the elements of the week days list
    public static final String V_SUNDAY    = "SU";
    public static final String V_MONDAY    = "MO";
    public static final String V_TUESDAY   = "TU";
    public static final String V_WEDNESDAY = "WE";
    public static final String V_THURSDAY  = "TH";
    public static final String V_FRIDAY    = "FR";
    public static final String V_SATURDAY  = "SA";

    public RecurrenceDetails()
    {
        super();
    }

    public RecurrenceDetails(HashMap hm)
    {
        for(int i=0; i<keysArray.length; i++)
            if(hm.containsKey(keysArray[i]))
                this.put(keysArray[i], hm.get(keysArray[i]));
    }

    public Object put(Object key, Object value)
    {
        Object oldValue = super.put(key, value);
        if(K_RULE_DATETIME_END.equals(key) && ((String)value).endsWith("Z"))
            super.put(K_TIMEZONE,
                    "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London");
        return oldValue;
    }

    public String getRRule()
    throws Exception
    {
        StringBuffer rRule = new StringBuffer();
        Object obj;

        obj = this.get(K_RULE_FREQUENCY);
        if(V_RULE_FREQUENCY_DAILY.equals(obj))
        {
            if(get(K_RULE_BYDAY)!=null)
                rRule.append(RRuleParser.FREQUENCY_WEEKLY);
            else rRule.append(RRuleParser.FREQUENCY_DAILY);
            obj = this.get(K_RULE_INTERVAL);
            if(obj!= null) rRule.append(obj);
            else throw new Exception("Recurrence interval not set!");
            if(get(K_RULE_BYDAY)!=null)
                rRule.append(" ").append(this.getRRuleWeekDaysList());
        }
        if(V_RULE_FREQUENCY_WEEKLY.equals(obj))
        {
            rRule.append(RRuleParser.FREQUENCY_WEEKLY);
            obj = this.get(K_RULE_INTERVAL);
            if(obj== null) throw new Exception("Recurrence interval not set!");
            rRule.append(obj).append(" ").append(this.getRRuleWeekDaysList());
        }
        if(V_RULE_FREQUENCY_MONTHLY.equals(obj))
        {
            if(this.get(K_RULE_BYDAY)==null && this.get(K_RULE_BYMONTHDAY)==null)
                throw new Exception("Not enought data for a monthly recurrence!");
            if(this.get(K_RULE_BYDAY)!=null)
            {
                rRule.append(RRuleParser.FREQUENCY_MONTHLY_BY_POSITION);
                obj = this.get(K_RULE_INTERVAL);
                if(obj== null) throw new Exception("Recurrence interval not set!");
                rRule.append(obj).append(" ").append(this.getRRuleDayOccurrenceList());
            }
            else
            {
                rRule.append(RRuleParser.FREQUENCY_MONTHLY_BY_DAY);
                obj = this.get(K_RULE_INTERVAL);
                if(obj== null) throw new Exception("Recurrence interval not set!");
                rRule.append(obj).append(" ").append(this.getRRuleOccurrenceList());
            }
        }
        if(V_RULE_FREQUENCY_YEARLY.equals(obj))
        {
            if(this.get(K_RULE_BYMONTH)==null && this.get(K_RULE_BYYEARDAY)==null)
                throw new Exception("Not enought data for a yearly recurrence!");
            if(this.get(K_RULE_BYMONTH)!=null)
            {
                rRule.append(RRuleParser.FREQUENCY_YEARLY_BY_MONTH);
                obj = this.get(K_RULE_INTERVAL);
                if(obj== null) throw new Exception("Recurrence interval not set!");
                rRule.append(obj).append(" ").append(this.getRRuleMonthsList());
            }
            else
            {
                rRule.append(RRuleParser.FREQUENCY_YEARLY_BY_DAY);
                obj = this.get(K_RULE_INTERVAL);
                if(obj== null) throw new Exception("Recurrence interval not set!");
                rRule.append(obj).append(" ").append(this.getRRuleYearDaysList());
            }
        }

        String numOccurrences = getNumOccurrences();
        String endDate = getRRuleEndDate();
        if(!"".equals(numOccurrences)) rRule.append(" ").append(numOccurrences);
        if(!"".equals(endDate)) rRule.append(" ").append(getRRuleEndDate());
        return rRule.toString();
    }



    public String toString()
    {
        String add_exclude  = (String)this.get(K_ADD_EXCLUDE);
        String timeZone     = (String)this.get(K_TIMEZONE);
        String date         = (String)this.get(K_DATETIME_START);
        String freq         = (String)this.get(K_RULE_FREQUENCY);
        String until        = (String)this.get(K_RULE_DATETIME_END);
        String count        = (String)this.get(K_RULE_COUNT);
        String interval     = (String)this.get(K_RULE_INTERVAL);
        //String[] bySecond   = (String[]) this.get(K_RULE_BYSECOND);
        //String[] byMinute   = (String[]) this.get(K_RULE_BYMINUTE);
        //String[] byHour     = (String[]) this.get(K_RULE_BYHOUR);
        HashMap[] byDay     = (HashMap[]) this.get(K_RULE_BYDAY);
        String[] byMonthDay = (String[]) this.get(K_RULE_BYMONTHDAY);
        String[] byYearDay  = (String[]) this.get(K_RULE_BYYEARDAY);
        //String[] byWeekNo   = (String[]) this.get(K_RULE_BYWEEKNO);
        String[] byMonth    = (String[]) this.get(K_RULE_BYMONTH);
        //String[] bySetPos   = (String[]) this.get(K_RULE_BYSETPOS);
        //String[] wkst       = (String[]) this.get(K_RULE_WKST);


        StringBuffer xml = new StringBuffer();
        if(add_exclude != null)
        {
            xml.append("<").append(this.get(K_ADD_EXCLUDE)).append(">");
            if(date!=null) xml.append("<date d=\"").append(date).append("\" tz=\"").
                                append(nullToVoid(timeZone)).append("\"/>");
            if(freq!=null) xml.append("<rule freq=\"").append(freq).append("\">");
            if(until!=null) xml.append("<until d=\"").append(until).append("\" tz=\"").
                                append(nullToVoid(timeZone)).append("\"/>");
            if(count!=null) xml.append("<count num=\"").append(count).append("\"/>");
            if(interval!=null)xml.append("<interval ival=\"").append(interval).append("\"/>");

            if(byDay!=null && byDay.length>0)
            {
                xml.append("<byday>");
                for(int i=0; i<byDay.length; i++)
                {
                    String day   = (String)byDay[i].get(K_RULE_BYDAY_DAY);
                    String ordwk = (String)byDay[i].get(K_RULE_BYDAY_ORDWK);
                    xml.append("<wkday ");
                    if(ordwk!=null) xml.append("ordwk=\"").append(ordwk).append("\" ");
                    xml.append("day=\"").append(nullToVoid(day)).append("\"/>");
                }
                xml.append("</byday>");
            }

            if(byMonthDay!=null && byMonthDay.length>0)
            {
                xml.append("<bymonthday modaylist=\"");
                for(int i=0; i<byMonthDay.length; i++)
                {
                    if(i>0) xml.append(",");
                    xml.append(byMonthDay[i]);

                }
                xml.append("\"/>");
            }

            if(byMonth!=null && byMonth.length>0)
            {
                xml.append("<bymonth molist=\"");
                for(int i=0; i<byMonth.length; i++)
                {
                    if(i>0) xml.append(",");
                    xml.append(byMonth[i]);

                }
                xml.append("\"/>");
            }
            if(byYearDay!=null && byYearDay.length>0)
            {
                xml.append("<byyearday yrdaylist=\"");
                for(int i=0; i<byYearDay.length; i++)
                {
                    if(i>0) xml.append(",");
                    xml.append(byYearDay[i]);

                }
                xml.append("\"/>");
            }
            if(freq!=null) xml.append("</rule>");
            xml.append("</").append(add_exclude).append(">");
        }
        return xml.toString();
    }

    private String getRRuleWeekDaysList()
    {
        HashMap[] byDay = (HashMap[])this.get(K_RULE_BYDAY);
        if(byDay == null) return "";

        StringBuffer weekDaysList = new StringBuffer();

        for(int i=0; i<byDay.length; i++)
        {
            if(i>0) weekDaysList.append(" ");
            String day=(String)byDay[i].get(K_RULE_BYDAY_DAY);
            if(V_SUNDAY.equals(day)) weekDaysList.append(RRuleParser.SUNDAY);
            if(V_MONDAY.equals(day)) weekDaysList.append(RRuleParser.MONDAY);
            if(V_TUESDAY.equals(day)) weekDaysList.append(RRuleParser.TUESDAY);
            if(V_WEDNESDAY.equals(day)) weekDaysList.append(RRuleParser.WEDNESDAY);
            if(V_THURSDAY.equals(day)) weekDaysList.append(RRuleParser.THURSDAY);
            if(V_FRIDAY.equals(day)) weekDaysList.append(RRuleParser.FRIDAY);
            if(V_SATURDAY.equals(day)) weekDaysList.append(RRuleParser.SATURDAY);
        }
        return weekDaysList.toString();
    }

    private String getRRuleDayOccurrenceList()
    {
        HashMap[] byDay = (HashMap[])this.get(K_RULE_BYDAY);
        if(byDay == null) return "";

        StringBuffer dayOccList = new StringBuffer();

        for(int i=0; i<byDay.length; i++)
        {
            if(i>0) dayOccList.append(" ");
            String day   = (String)byDay[i].get(K_RULE_BYDAY_DAY);
            String ordwk = (String)byDay[i].get(K_RULE_BYDAY_ORDWK);
            if(ordwk!=null) dayOccList.append(signLast(ordwk)).append(" ");
            if(V_SUNDAY.equals(day)) dayOccList.append(RRuleParser.SUNDAY);
            if(V_MONDAY.equals(day)) dayOccList.append(RRuleParser.MONDAY);
            if(V_TUESDAY.equals(day)) dayOccList.append(RRuleParser.TUESDAY);
            if(V_WEDNESDAY.equals(day)) dayOccList.append(RRuleParser.WEDNESDAY);
            if(V_THURSDAY.equals(day)) dayOccList.append(RRuleParser.THURSDAY);
            if(V_FRIDAY.equals(day)) dayOccList.append(RRuleParser.FRIDAY);
            if(V_SATURDAY.equals(day)) dayOccList.append(RRuleParser.SATURDAY);
        }
        return dayOccList.toString();
    }

    private String getRRuleOccurrenceList()
    {
        StringBuffer occList = new StringBuffer();

        String[] byMonthDay = (String[]) this.get(K_RULE_BYMONTHDAY);
        for(int i=0; i<byMonthDay.length; i++)
        {
            if(i>0) occList.append(" ");
            occList.append(signLast(byMonthDay[i]));
        }
        return occList.toString();
    }

    private String getRRuleMonthsList()
    {
        StringBuffer months = new StringBuffer();
        String[] byMonth = (String[]) this.get(K_RULE_BYMONTH);
        for(int i=0; i<byMonth.length; i++)
        {
            if(i>0) months.append(" ");
            months.append(signLast(byMonth[i]));
        }
        return months.toString();
    }

    private String getRRuleYearDaysList()
    {
        StringBuffer months = new StringBuffer();
        String[] byYearDay = (String[]) this.get(K_RULE_BYYEARDAY);
        for(int i=0; i<byYearDay.length; i++)
        {
            if(i>0) months.append(" ");
            months.append(byYearDay[i]);
        }
        return months.toString();
    }

    private String getNumOccurrences()
    {
        if(this.get(K_RULE_COUNT) != null)
            return "#"+this.get(K_RULE_COUNT);
        if(this.get(K_RULE_DATETIME_END) == null) return "#0";
        return "";
    }

    private String getRRuleEndDate()
    throws Exception
    {
        String endDate  = (String)this.get(K_RULE_DATETIME_END);

        if(endDate==null) return "";

        String timeZone = (String)this.get(K_TIMEZONE);

        if(endDate.length()==9)
            if(endDate.endsWith("Z"))
                endDate =
                    endDate.substring(0, endDate.length()-1) +
                    "T235959Z";
            else throw new Exception("Invalid date: " + endDate);

        if(endDate.length()<9)
            endDate = endDate + "T235959";

        if(timeZone!=null)
            return UtilityCalendar.covertLocalDateToGMT(endDate, timeZone);
        else
            if(endDate.endsWith("Z")) return endDate;
            else return endDate+"Z";
    }

    private String signLast(String str)
    {
        if(str==null) return "";
        if(str.startsWith("-") || str.startsWith("+"))
            return str.substring(1)+str.charAt(0);
        else return str;
    }

    private static Object nullToVoid(Object obj)
    {
        if(obj == null && "null".equals(obj)) return "";
        else return obj;
    }

}
